"""
reaper osc communicator
type /play 1 to make reaper play
type quit to dump all received data to 'reaper_osc.txt'

copyright.2012@barskdata.net
use at own risk - freely as long as copyright stays in

tested only on :
    python 2.7.2
    windows xp professional 
    reaper 4.22

--------------------
setup manual:

install python 2.7
easy_install pyOSC or 
pip install pyOSC

Start Reaper

    -> Ctrl+P 
    -> Control Surfaces 
    -> Add 
    -> OSC(Open Sound Control) 
    -> Edit
        Receive on port 8000
        Send to port 9000 , Device IP 127.0.0.1
        Disallow binding
    -> Pattern config
    -> Open Config Directory
        make a copy of Default.ReaperOSC
        call it pylotech.ReaperOSC
    -> Modify pylotech.ReaperOSC like

        REAPER_TRACK_FOLLOWS REAPER
        DEVICE_TRACK_FOLLOWS LAST_TOUCHED
        DEVICE_TRACK_BANK_FOLLOWS MIXER

       in order to receive as much as possible

Usage:

Start this script in a terminal (python pylotech.py)
Fool around in reaper, adding tracks, play, record or whatever
type /play 1 in terminal to make reaper play
type /stop in terminal to make reaper stop
type quit to dump all received data to 'reaper_osc.txt'


--------------------

"""
import threading
import OSC

class Server(threading.Thread):
    def discovery_handler(self,path,container):
        #print 'just discovered', path, container
        print '.',

    def reaper_handler(self, addr, tags, data, source):
        # is an illegal character in osc address
        # - so with that in front we avoid name clashes
        #m: monitors
        #t: tags
        #d: data


        # collect all reaper data and celebrate each new discovery
        # with a call to discovery_handler
        slot = self.reaper
        addr_parts = addr.split('/')[1:]
        for p in addr_parts:
            if not p in slot:
                slot[p]={'#m':self.discovery_handler(addr_parts,slot)}
            slot = slot[p]
        slot['#d'] = data
        slot['#t'] = tags

    def run(self):
        HOST, PORT = "localhost", 9000
        self.reaper = {}
        self.selected_track = 0
        self.server = OSC.OSCServer((HOST, PORT))
        self.server.print_tracebacks = True
        self.server.addDefaultHandlers()
        self.server.addMsgHandler('default', self.reaper_handler)

        # run the server
        self.server.serve_forever()

    def close(self):
        self.server.close()

class Client:
    def __init__(self):
        HOST, PORT = "localhost", 8000
        self.client = OSC.OSCClient()
        self.client.connect((HOST,PORT))

    def send(self, data):
        strdata = data.split(' ')
        print strdata
        address = strdata[0]
        typeddata = [] 
        for i in strdata[1:]:
            if i.isdigit():
                typeddata.append(int(i))
                continue
            try:
                f = float(i)
                typeddata.append(f)
                continue
            except:
                pass
            typeddata.append(i)

        m = OSC.OSCMessage(address)
        [m.append(d) for d in typeddata]
        self.client.send(m)

    def close(self):
        self.client.close()

def main():
    server = Server()
    server.daemon = True
    server.start()
    client = Client()
    data = "/hello world of reaper@barskdata.net"
    while data != "quit":
        client.send(data)
        data = raw_input()
    import pprint
    import codecs
    codecs.open('reaper_osc.txt','w','utf-8').write(pprint.pformat(server.reaper, indent=2))


if __name__=='__main__':
   main() 
        
